import { useEffect, useState, useRef } from "react";
import { connect, useDispatch } from "react-redux";
import { Link } from "react-router-dom";
import { actionGoodsFind } from "actions";
import { actionPromiseClear, store } from "reducers";
import SearchInput from "./SearchInput";
import { CSearchResults } from ".";

// import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
// import { faSearch } from "@fortawesome/free-solid-svg-icons";

export const SearchBar = ({
    onSearch,
    onSearchButtonClick,
    render = null,
    renderParams = {},
    searchLink = "/search/",
} = {}) => {
    const ref = useRef();
    const [inputValue, setInputValue] = useState("");
    const [isChildrenOpen, setIsChildrenOpen] = useState(false);
    const [inputTimeout, setInputTimeout] = useState(null);
    const R = render;

    useEffect(() => {
        if (inputTimeout) {
            clearTimeout(inputTimeout);
        }

        const checkClickOutsideHeaderSearchBar = (e) => {
            if (ref.current && !ref.current.contains(e.target)) {
                setIsChildrenOpen(false);
            } else {
                inputValue.length && setIsChildrenOpen(true);
            }
        };

        inputValue && onSearch(inputValue);
        setIsChildrenOpen(!!inputValue?.length);
        document.addEventListener("mousedown", checkClickOutsideHeaderSearchBar);
        return () => {
            document.removeEventListener("mousedown", checkClickOutsideHeaderSearchBar);
        };
    }, [inputValue]);

    return (
        <div className={`SearchBar ${!isChildrenOpen && "hide"}`} ref={ref}>
            <SearchInput
                onChange={(text) => {
                    setInputValue(text);
                }}
                value={inputValue}
            />
            {!!inputValue ? (
                <Link
                    className="Link SearchButton"
                    to={`${searchLink}${inputValue}`}
                    onClick={() => {
                        setInputValue("");
                        onSearchButtonClick();
                    }}
                >
                    Поиск
                </Link>
            ) : (
                <span className="Link SearchButton">Поиск</span>
            )}
            {isChildrenOpen && (
                <R
                    onItemClick={() => {
                        setInputValue("");
                        onSearchButtonClick();
                    }}
                    {...renderParams}
                />
            )}
        </div>
    );
};

export const CSearchBar = connect(null, {
    onSearch: (text) => actionGoodsFind({ text, limit: 5 }),
    onSearchButtonClick: () => actionPromiseClear("goodsFind"),
})(SearchBar);
