import { connect } from "react-redux";
import React, { useState, useEffect } from "react";
import { Oval } from "react-loader-spinner";
import Select from "react-select";
import { actionCategoryUpdate } from "actions/actionCategoryUpdate";
import { actionPromise, actionPromiseClear } from "reducers";
import { SuccessIcon } from "components/SuccessIcon";
import { FailedIcon } from "components/FailedIcon";
import { actionGoodUpdate } from "actions/actionGoodUpdate";
import { EntityEditor } from "components/EntityEditor";
import { actionUploadFile } from "actions";
import { actionUploadFiles } from "actions/actionUploadFiles";

const CGoodEditor = connect(
    (state) => ({
        entity: state.promise?.adminGoodById?.payload || {},
        uploadFiles: state.promise?.uploadFiles,
    }),
    {
        onFileDrop: (files) => actionUploadFiles(files),
    }
)(EntityEditor);

export const GoodForm = ({
    serverErrors,
    onSaveClick,
    onSave,
    onClose,
    promiseStatus,
    catList = [],
    good = {},
} = {}) => {
    const defaultErrorSchema = {
        name: [],
        description: [],
        images: [],
        categories: [],
        price: [],
    };
    const [inputName, setInputName] = useState("");
    const [inputDescription, setInputDescription] = useState("[]");
    const [inputPrice, setInputPrice] = useState("");
    const [inputCategories, setInputCategories] = useState([]);
    const [inputImages, setInputImages] = useState([]);

    const [isValid, setIsValid] = useState(false);
    const [errorSchema, setErrorSchema] = useState(defaultErrorSchema);

    useEffect(() => {
        setInputName(good?.name || "");
        setInputDescription(good?.description || "");
        setInputPrice(good?.price || "");
        setInputCategories(good?.categories || []);
    }, [good]);

    useEffect(() => {
        let valid = true;
        for (let errorList of Object.values(errorSchema)) {
            errorList?.length > 0 && (valid = false);
        }

        setIsValid(valid);
    }, [errorSchema]);

    useEffect(() => {
        return () => {
            onClose && onClose();
            console.log(1);
        };
    }, []);
    return (
        <div className="CategoryForm has-validation">
            {(serverErrors || []).map((error) => (
                <div className="text-danger">{error?.message}</div>
            ))}

            <div className="mb-3 has-validation">
                <label className="form-label">Название</label>

                <input
                    className={`form-control ${errorSchema["name"].length > 0 && `is-invalid`}`}
                    id="name"
                    name="name"
                    placeholder="Name"
                    value={inputName}
                    onChange={(e) => setInputName(e.target.value)}
                />
                {(errorSchema["name"] || []).map((error) => (
                    <div className="invalid-feedback ">{error}</div>
                ))}
            </div>

            <div className="mb-3 has-validation">
                <label className="form-label">Картинки</label>
                <CGoodEditor onImagesSave={(images) => setInputImages(images)} />
                {(errorSchema["images"] || []).map((error) => (
                    <div className="invalid-feedback ">{error}</div>
                ))}
            </div>

            <div className="mb-3 has-validation">
                <label className="form-label">Описание</label>
                <textarea
                    className={`form-control ${errorSchema["description"].length > 0 && `is-invalid`}`}
                    placeholder="Описание"
                    value={inputDescription}
                    onChange={(e) => setInputDescription(e.target.value)}
                />

                {(errorSchema["description"] || []).map((error) => (
                    <div className="invalid-feedback ">{error}</div>
                ))}
            </div>

            <div className="mb-3 has-validation">
                <label className="form-label">Цена</label>
                <input
                    className={`form-control ${errorSchema["price"].length > 0 && `is-invalid`}`}
                    type="number"
                    placeholder="0.00"
                    value={inputPrice}
                    onChange={(e) => setInputPrice(e.target.value)}
                />

                {(errorSchema["subCatergories"] || []).map((error) => (
                    <div className="invalid-feedback ">{error}</div>
                ))}
            </div>

            <div className="mb-3 has-validation">
                <label className="form-label">Категории</label>
                <Select
                    value={inputCategories.map(({ _id, name }) => ({ value: _id, label: name }))}
                    closeMenuOnSelect={false}
                    onChange={(e) => setInputCategories(e.map(({ label, value }) => ({ _id: value, name: label })))}
                    options={catList?.map(({ _id, name }) => ({ value: _id, label: name }))}
                    isMulti={true}
                />

                {(errorSchema["categories"] || []).map((error) => (
                    <div className="invalid-feedback ">{error}</div>
                ))}
            </div>

            <div className="confirmButton">
                <button
                    disabled={!isValid}
                    className="btn btn-success "
                    onClick={() => {
                        let goodToSave = {};
                        good?._id && (goodToSave._id = good._id);
                        goodToSave.name = inputName;
                        goodToSave.description = inputDescription;
                        goodToSave.price = +inputPrice;
                        goodToSave.categories = inputCategories;
                        console.log(inputImages);
                        goodToSave.images = inputImages?.map(({ _id }) => ({ _id })) || [];

                        onSaveClick && onSaveClick();
                        onSave(goodToSave);
                    }}
                >
                    Сохранить
                </button>
                <div className="status">
                    {!!promiseStatus && promiseStatus === "PENDING" ? (
                        <Oval color="#000" height={20} width={20} />
                    ) : promiseStatus === "FULFILLED" ? (
                        <SuccessIcon color="#0F0" fontSize={30} />
                    ) : promiseStatus === "REJECTED" ? (
                        <FailedIcon color="#F00" fontSize={30} />
                    ) : (
                        ""
                    )}
                </div>{" "}
            </div>
        </div>
    );
};

// const CRegisterForm = connect((state) => ({ serverErrors: state.promise?.register?.error || [] }), {
//     onRegister: (login, password) => actionRegister(login, password),
// })(RegisterForm);

export const CGoodForm = connect(
    (state) => ({
        catList: state.promise.catAll?.payload || [],
        promiseStatus: state.promise.goodUpsert?.status || null,
    }),
    {
        onSave: (good) => actionGoodUpdate(good),
        onClose: () => actionPromiseClear("goodUpsert"),
    }
)(GoodForm);

// const CGoodEditor = connect(
//     (state) => ({
//         entity: state.promise?.good?.payload,
//         uploadFile: state.promise?.uploadFile,
//     }),
//     {
//         onFileDrop: (file) => actionUploadFile(file),
//         onSave: (_id, images) => actionUpdateGood({ _id, images: images.map((img) => ({ _id: img._id })) }),
//     }
// )(EntityEditor);
