import { DropZone } from "components/DropZone";
import { connect } from "react-redux";
import { CProfileImage } from "components/ProfileImage";
import { actionUpdateAvatar } from "actions";
import { useState, useEffect } from "react";
import { actionUserUpdate } from "actions/actionUserUpdate";
import { ProfileFormInput } from "./ProfileFormInput";
import { FaEdit } from "react-icons/fa";

const CDropZone = connect(null, { onFileDrop: (acceptedFiles) => actionUpdateAvatar(acceptedFiles[0]) })(DropZone);

export const ProfileForm = ({ profile, onProfileSave }) => {
    const { nick, createdAt, login } = profile || {};
    const [isNickChanging, setIsNickChanging] = useState(false);

    const [isLetterShown, setIsLetterShown] = useState(false);

    return (
        <div className="ProfileForm">
            <div className="row">
                <div className="col-5 col-md-4 col-lg-3">
                    <CDropZone>
                        <div
                            className="profileImageWrapper"
                            onMouseEnter={() => setIsLetterShown(true)}
                            onMouseLeave={() => setIsLetterShown(false)}
                        >
                            <CProfileImage />
                            <div className={`letter ${isLetterShown && "show"}`}>Drop file or click to update</div>
                        </div>
                    </CDropZone>
                </div>
                <div className="col-7  col-md-8 col-lg-9">
                    <table>
                        <tbody>
                            <tr>
                                <td>Имя пользоватиля:</td>
                                <td>
                                    {isNickChanging ? (
                                        <ProfileFormInput
                                            value={nick}
                                            onSave={(newNick) => {
                                                newNick.length && newNick !== nick && onProfileSave({ nick: newNick });
                                                setIsNickChanging(false);
                                            }}
                                        />
                                    ) : (
                                        nick
                                    )}
                                </td>
                                <td className="editButtonWrapper">
                                    {!isNickChanging && (
                                        <span className="editButton" onClick={() => setIsNickChanging(true)}>
                                            <FaEdit />
                                        </span>
                                    )}
                                </td>
                            </tr>
                            <tr>
                                <td>Дата регистрации:</td>
                                <td>
                                    {!!createdAt && (
                                        <div>
                                            {`00${new Date(+createdAt).getDay()}`.slice(-2)}/
                                            {`00${new Date(+createdAt).getMonth()}`.slice(-2)}/
                                            {`0000${new Date(+createdAt).getFullYear()}`.slice(-4)}
                                        </div>
                                    )}
                                </td>
                                <td></td>
                            </tr>
                            <tr>
                                <td>Логин:</td>
                                <td>{login}</td>
                                <td></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    );
};

export const CProfileForm = connect((state) => ({ profile: state.promise?.aboutMe?.payload }), {
    onProfileSave: (profile = {}) => actionUserUpdate(profile),
})(ProfileForm);
