import { useEffect, useState } from "react";
import { backendURL } from "helpers";
import CBuyButton from "components/BuyButton";
import { connect } from "react-redux";
import { actionCartDelete } from "reducers";
import defaultGoodImage from "images/defaultGoodImage.png";

export const GoodDetail = ({ good, cart, token }) => {
    const { _id = "", name = "", price = "", description = "", images = [] } = good || {};
    const [inCart, setInCart] = useState(0);

    useEffect(() => {
        setInCart((cart[_id] && cart[_id].count) || 0);
    }, [_id, cart]);

    return (
        <div className="GoodDetail page">
            <div className="row">
                <div className="col-4">
                    <img src={images[0]?.url ? `${backendURL}/${images ? images[0]?.url : ""}` : defaultGoodImage} />
                </div>
                <div className="col-8">
                    <div className="w-100">
                        <h3>{name}</h3>
                        <span className="price">
                            <b>Цена:</b> {price}
                        </span>
                    </div>
                    <div className="w-100 desc">
                        <b>Описание:</b>
                        <br />
                        <span className="name">{description}</span>
                    </div>
                    {token ? (
                        <div className="w-100 d-flex">
                            <CBuyButton good={{ name, images, price, _id }} />
                        </div>
                    ) : (
                        ""
                    )}
                </div>
            </div>
        </div>
    );
};

const CGoodDetail = connect((state) => ({
    good: state.promise.goodById?.payload || {},
    cart: state.cart || {},
    token: state.auth?.token || null,
}))(GoodDetail);

export default CGoodDetail;
