import { Route, Switch } from "react-router-dom";
import { CAdminCategoryList } from "./AdminCategory";
import { CCategoryTree } from "./CategoryTree";
import { useEffect } from "react";
import { connect } from "react-redux";
import { actionFeedAdd, actionFeedCats, actionFeedClear, actionFeedGoods, actionPromiseClear, store } from "reducers";
import { actionCatById, actionGoodById } from "actions";
import { CAdminCategoryDetail } from "./AdminCategory/AdminCategoryDetail";
import { CAdminGoodDetail, CAdminGoodList } from "./AdminGoods";
import { actionFeedGoodsFind, actionFeedCatsFind } from "../../reducers";

const AdminCategoryListContainer = ({ cats }) => {
    useEffect(() => {
        store.dispatch(actionFeedCats(cats?.length || 0));
        window.onscroll = (e) => {
            if (window.innerHeight + window.scrollY >= document.body.offsetHeight) {
                const {
                    feed,
                    promise: { feedCatAll },
                } = store.getState();
                if (feedCatAll.status !== "PENDING") {
                    store.dispatch(actionFeedCats(feed.payload?.length || 0));
                }
            }
        };
        return () => {
            store.dispatch(actionFeedClear());
            store.dispatch(actionPromiseClear("feedCatAll"));

            store.dispatch(actionPromiseClear("categoryUpsert"));
            window.onscroll = null;
        };
    }, []);

    useEffect(() => {
        if (cats.length) store.dispatch(actionFeedAdd(cats));
    }, [cats]);
    return <CAdminCategoryList />;
};

const AdminGoodListContainer = ({ goods }) => {
    useEffect(() => {
        store.dispatch(actionFeedGoods());
        window.onscroll = (e) => {
            if (window.innerHeight + window.scrollY >= document.body.offsetHeight) {
                const {
                    feed,
                    promise: { feedGoodsAll },
                } = store.getState();
                if (feedGoodsAll.status !== "PENDING") {
                    store.dispatch(actionFeedGoods(feed.payload?.length || 0));
                }
            }
        };
        return () => {
            store.dispatch(actionFeedClear());
            store.dispatch(actionPromiseClear("feedGoodsAll"));
            store.dispatch(actionPromiseClear("goodUpsert"));
            window.onscroll = null;
        };
    }, []);

    useEffect(() => {
        if (goods.length) store.dispatch(actionFeedAdd(goods));
    }, [goods]);
    return <CAdminGoodList />;
};

const AdminGoodListSearchContainer = ({
    goods,
    match: {
        params: { searchData },
    },
}) => {
    useEffect(() => {
        store.dispatch(actionFeedClear());
        store.dispatch(actionFeedGoodsFind({ text: searchData }));
        window.onscroll = (e) => {
            if (window.innerHeight + window.scrollY >= document.body.offsetHeight) {
                const {
                    feed,
                    promise: { feedGoodsFind },
                } = store.getState();
                if (feedGoodsFind.status !== "PENDING") {
                    store.dispatch(actionFeedGoodsFind({ skip: feed.payload?.length || 0, text: searchData }));
                }
            }
        };
        return () => {
            store.dispatch(actionFeedClear());
            store.dispatch(actionPromiseClear("feedGoodsFind"));
            store.dispatch(actionPromiseClear("goodUpsert"));
            window.onscroll = null;
        };
    }, []);

    useEffect(() => {
        if (goods.length) store.dispatch(actionFeedAdd(goods));
    }, [goods]);
    return <CAdminGoodList />;
};

const AdminCategoryListSearchContainer = ({
    cats,
    match: {
        params: { searchData },
    },
}) => {
    useEffect(() => {
        store.dispatch(actionFeedClear());
        store.dispatch(actionFeedCatsFind({ text: searchData }));
        window.onscroll = (e) => {
            if (window.innerHeight + window.scrollY >= document.body.offsetHeight) {
                const {
                    feed,
                    promise: { feedCatsFind },
                } = store.getState();
                if (feedCatsFind.status !== "PENDING") {
                    store.dispatch(actionFeedCatsFind({ skip: feed.payload?.length || 0, text: searchData }));
                }
            }
        };
        return () => {
            store.dispatch(actionFeedClear());
            store.dispatch(actionPromiseClear("feedCatsFind"));
            store.dispatch(actionPromiseClear("categoryUpsert"));
            window.onscroll = null;
        };
    }, []);

    useEffect(() => {
        if (cats.length) store.dispatch(actionFeedAdd(cats));
    }, [cats]);
    return <CAdminCategoryList />;
};

const AdminCategoryDetailContainer = ({
    match: {
        params: { _id },
    },
}) => {
    useEffect(() => {
        if (_id) {
            store.dispatch(actionCatById(_id, "adminCatById"));
        } else {
            store.dispatch(actionPromiseClear("adminCatById"));
        }
    }, [_id]);
    return <CAdminCategoryDetail />;
};

const AdminGoodDetailContainer = ({
    match: {
        params: { _id },
    },
}) => {
    useEffect(() => {
        if (_id) {
            store.dispatch(actionGoodById({ _id, promiseName: "adminGoodById" }));
        } else {
            store.dispatch(actionGoodById("adminGoodById"));
        }
    }, [_id]);
    return <CAdminGoodDetail />;
};

const CAdminCategoryListContainer = connect((state) => ({ cats: state.promise?.feedCatAll?.payload || [] }))(
    AdminCategoryListContainer
);

const CAdminGoodListContainer = connect((state) => ({ goods: state.promise?.feedGoodsAll?.payload || [] }))(
    AdminGoodListContainer
);

const CAdminGoodListSearchContainer = connect((state) => ({ goods: state.promise?.feedGoodsFind?.payload || [] }))(
    AdminGoodListSearchContainer
);

const CAdminCategoryListSearchContainer = connect((state) => ({ cats: state.promise?.feedCatsFind?.payload || [] }))(
    AdminCategoryListSearchContainer
);

export const AdminPanel = () => {
    return (
        <div className="AdminPanel page">
            <div className="body">
                <Switch>
                    <Route path="/admin/" component={CCategoryTree} exact />
                    <Route path="/admin/categories/" component={CAdminCategoryListContainer} exact />
                    <Route path="/admin/goods/" component={CAdminGoodListContainer} exact />
                    <Route path="/admin/goods/search/:searchData/" component={CAdminGoodListSearchContainer} exact />
                    <Route
                        path="/admin/category/search/:searchData/"
                        component={CAdminCategoryListSearchContainer}
                        exact
                    />
                    <Route path="/admin/category/:_id?" component={AdminCategoryDetailContainer} />
                    <Route path="/admin/good/:_id?" component={AdminGoodDetailContainer} exact />
                </Switch>
            </div>
        </div>
    );
};
